def get_youtube_id(youtube_url: str):
    """Get the YouTube video ID from a URL

    Parameters:
        youtube_url (str): a YouTube video URL

    >>> get_youtube_id('https://www.youtube.com/watch?v=dQw4w9WgXcQ')
    "dQw4w9WgXcQ"
    >>> get_youtube_id('https://www.youtube.com/watch?v=dQw4w9WgXcQ&otherparams=anything')
    "dQw4w9WgXcQ"
    >>> get_youtube_id('youtu.be/dQw4w9WgXcQ')
    "dQw4w9WgXcQ"
    >>> get_youtube_id('dQw4w9WgXcQ')
    "dQw4w9WgXcQ"
    """

    if "youtube.com" in youtube_url:
        # Regular URL, need to check and other queries in URL
        video_id = youtube_url.split("?v=")[1].split("&")[0]

    elif "youtu.be" in youtube_url:
        # Shortened URL
        video_id = youtube_url.split("/")[1]

    elif len(youtube_url) == 11:
        # Each video ID is 11 characters
        video_id = youtube_url

    else:
        raise Exception("Unable to find YouTube video ID")

    return video_id



from .video_id import get_youtube_id


def test_video_id():
    assert (
        get_youtube_id("https://www.youtube.com/watch?v=dQw4w9WgXcQ") == "dQw4w9WgXcQ"
    )
    assert (
        get_youtube_id(
            "https://www.youtube.com/watch?v=dQw4w9WgXcQ&otherparams=anything"
        )
        == "dQw4w9WgXcQ"
    )

    assert get_youtube_id("youtu.be/dQw4w9WgXcQ") == "dQw4w9WgXcQ"

    assert get_youtube_id("dQw4w9WgXcQ") == "dQw4w9WgXcQ"



from .date import DateCode
from .date import get_date


def test_get_date():
    assert (
        get_date(
            [DateCode.DAY, DateCode.MONTH, DateCode.COMMA, DateCode.YEAR],
            return_strf=True,
        )
        == "%-d %B, %Y"
    )

    assert (
        get_date(
            [
                DateCode.MONTH_NUMBER,
                DateCode.SLASH,
                DateCode.DAY,
                DateCode.SLASH,
                DateCode.SHORT_YEAR,
            ],
            return_strf=True,
        )
        == "%-m/%-d/%y"
    )



from datetime import datetime
from enum import Enum
from typing import List


class DateCode(Enum):
    SHORT_DAY_NAME = "%a"
    DAY_NAME = "%A"
    DAY = "%-d"
    ZERO_PADDED_DAY = "%-d"
    SHORT_MONTH = "%b"
    MONTH = "%B"
    MONTH_NUMBER = "%-m"
    YEAR = "%Y"
    SHORT_YEAR = "%y"
    COMMA = ","
    SLASH = "/"


def get_date(order: List[DateCode], return_strf=False) -> str:
    """Get the date using codes from DateCode, or a strftime builder

    Parameters:
        order (List[DateCode]): the date format
        return_strf (bool, default False): use to return the datetime `strftime` format.

    Use return_strf for testing.
    """

    now = datetime.now()

    string = ""
    for index, code in enumerate(order):
        try:
            next_code = order[index + 1]
        except:
            next_code = None

        string += code.value

        if next_code not in (DateCode.COMMA, DateCode.SLASH) and code != DateCode.SLASH:
            string += " "

    if return_strf:
        return string.strip()

    return now.strftime(string).strip()



from youtube_dl import YoutubeDL

from .video_id import get_youtube_id


def download_video(url: str):
    """Download the video in the highest possible format in the same directory

    Parameters:
        url (str): the YouTube video URL or video ID
    """

    opts = {
        "format": "bestvideo+bestaudio",
        "outtmpl": "%(title)s.%(ext)s",
    }

    video_id = get_youtube_id(url)

    with YoutubeDL(opts) as ydl:
        ydl.download([f"https://www.youtube.com/watch?v={video_id}"])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['armorstand', 'armorstand.date', 'armorstand.youtube']

package_data = \
{'': ['*']}

install_requires = \
['youtube_dl>=2021.2.10,<2022.0.0']

setup_kwargs = {
    'name': 'armorstand',
    'version': '21.2.22.20.27',
    'description': 'All my utility functions',
    'long_description': '# Armorstand\n\n> **Util functions.**',
    'author': 'ninest',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ninest/armorstand/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
